#!/usr/bin/env python3

"""
Entrypoint for docker-droplet
Parse command line arguments and call the selected handlers
"""

import sys
from os import getcwd, path, environ
from inspect import cleandoc
from os.path import dirname, exists

from docopt import docopt  # type: ignore

# If packaged multi_job will be scoped, otherwise append parent path.
package_directory = path.realpath(path.join(__file__, "../.."))
sys.path.append(package_directory)

from docker_droplet.down import tear_down
from docker_droplet.exceptions import MissingVariable, PathNotResolvable
from docker_droplet.up import set_up

CLI = cleandoc(
    """
    Usage:
       docker-droplet up [options]
       docker-droplet down [options]
    
    Options:
       --droplet-name=<name>
       --ssh-key=<ssh-key>
       --token=<token>
       --project=<project>
       --domain=<domain>
       --config-path=<config-path>"""
)


class InputArg:
    def __init__(self, name: str, value: str) -> None:
        self.name = name
        self.value = None if value == "None" else value

    def assign_default(self, default: str) -> None:
        """
        If the object's value attribute is None the assign a default value
        
        Args:
            default (str): 
        """
        if not self.value:
            self.value = default

    def validate_path(self, check_file_exists: bool = False) -> None:
        """
        Check if path's directory is resolvable. Optionally check if the path itself exists.
        
        Args:
            check_file_exists (bool, optional): [description]. Defaults to False.
        
        Raises:
            PathNotResolvable: [description]
            PathNotResolvable: [description]
        """
        if not path.exists(path.dirname(self.value)):
            raise PathNotResolvable(self.name, self.value)

        if check_file_exists and not path.exists(self.value):
            raise PathNotResolvable(self.name, self.value)

    def sync_env(self) -> None:
        """
        Synchronize the object's value with the environment variable TF_VAR_<name>
        """
        NAME = "TF_VAR_DOCKER_DROPLET_" + self.name.upper()
        if self.value:
            environ.putenv(NAME, self.value)
        else:
            self.value = environ.get(NAME, None)

    def set_required(self) -> None:
        if not self.value:
            raise MissingVariable(self.name)

    def __str__(self) -> str:
        return f"{self.name}: {self.value}"


def main() -> None:
    """
    Entry point for docker-droplet
    """
    arguments = docopt(CLI)
    droplet_name = InputArg("droplet_name", arguments["--droplet-name"])
    ssh_key = InputArg("ssh_key", arguments["--ssh-key"])
    token = InputArg("token", arguments["--token"])
    project = InputArg("project", arguments["--project"])
    domain = InputArg("domain", arguments["--domain"])
    config_path = InputArg("config_path", arguments["--config-path"])

    token.sync_env()
    token.set_required()

    if arguments["up"]:
        config_path.assign_default("./config.tf")
        config_path.validate_path()

        droplet_name.sync_env()
        droplet_name.set_required()

        ssh_key.sync_env()
        ssh_key.set_required()
        ssh_key.validate_path(check_file_exists=True)

        set_up(
            droplet_name.value,
            ssh_key.value,
            token.value,
            project.value,
            domain.value,
            config_path.value,
        )

    if arguments["down"]:
        config_path.set_required()
        config_path.validate_path()
        tear_down(token.value, config_path.value)


if __name__ == "__main__":
    main()



from os.path import dirname
from subprocess import run


def tear_down(token, config_path) -> None:
    """
    Destroys the terraform infrastructure specified in the config path provided.
    
    Args:
        token ([type]): Digitalocean access token
        config_path ([type]): Terraform config path
    """
    run(["terraform", "destroy"], cwd=dirname(config_path))



class PathNotResolvable(Exception):
    def __init__(self, name, value) -> None:
        msg = f"The provided path for {name}: '{value}' is not resolvable"
        super().__init__(msg)


class MissingVariable(Exception):
    def __init__(self, name) -> None:
        msg = f"Variable {name} is missing"
        super().__init__(msg)



"""
Setup script for docker-droplet
"""

from setuptools import find_packages, setup
from distutils.core import setup, Command
from unittest import TestLoader, TextTestRunner
from sphinx.setup_command import BuildDoc

__version__ = "1.0.3"


class DocsCommand(BuildDoc):
    description = "Generate build configuration and make docs"

    def initialize_options(self) -> None:
        """  
        Docs command override. Call the parent initializer then add version and config directory
        """
        super().initialize_options()
        self.version = __version__
        self.config_dir = "./docker_droplet/docs"


class TestsCommand(Command):
    description = "Discover and run tests"
    user_options = []

    def initialize_options(self) -> None:
        pass

    def finalize_options(self) -> None:
        pass

    def run(self) -> None:
        """
        Discover and run tests        
        """
        suite = TestLoader().discover("./docker_droplet/tests")
        TextTestRunner().run(suite)


with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="docker-droplet",
    version=__version__,
    license="MIT",
    description="Create a single digital ocean droplet and provision it to run the docker engine over a simple cli",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JoelLefkowitz/docker-droplet",
    include_package_data=True,
    package_data={"": ["**/*yml", "**/*.jinja2"]},
    packages=find_packages(exclude=["tests"]),
    install_requires=[
        "docopts>=0.6.1",
        "ansible>=2.8.0",
        "jinja2>=2.11.1",
        "doboto>=0.6.1",
        "sphinx>=2.4.1"
    ],
    entry_points={"console_scripts": ["docker-droplet=docker_droplet.main:main"]},
    cmdclass={"docs": DocsCommand, "test": TestsCommand},
    python_requires=">= 3.6",
    author="Joel Lefkowitz",
    author_email="joellefkowitz@hotmail.com",
)



from os import path
from jinja2 import Environment, FileSystemLoader


def create_config(droplet_name, ssh_key, project, domain) -> str:
    """
    Create a terraform configuration file for a single droplet
    
    Args:
        droplet_name ([type]): 
        ssh_key ([type]):
        project ([type]):
        domain ([type]):
    
    Returns:
        str: Configuration file as a string
    """
    directory = path.realpath(path.join(__file__, ".."))
    templateLoader = FileSystemLoader(searchpath=directory)
    templateEnv = Environment(loader=templateLoader)

    print(templateEnv)

    providers_template = templateEnv.get_template("providers.jinja2")
    resources_template = templateEnv.get_template("digitalocean.jinja2")

    providers_config = providers_template.render()
    resources_config = resources_template.render(
        droplet_name=droplet_name, ssh_key=ssh_key, project=project, domain=domain
    )
    return "\n\n".join([providers_config, resources_config])
